@@ -19,12 +19,11 @@
 import abc
 import functools
 import time
-import uuid
 
 import enum
 from oslo_config import cfg
 from oslo_log import log as logging
-import six
+from oslo_utils import uuidutils
 
 from zaqar.common import decorators
 from zaqar.storage import errors
@@ -35,6 +34,7 @@ DEFAULT_QUEUES_PER_PAGE = 10
 DEFAULT_MESSAGES_PER_PAGE = 10
 DEFAULT_POOLS_PER_PAGE = 10
 DEFAULT_SUBSCRIPTIONS_PER_PAGE = 10
+DEFAULT_TOPICS_PER_PAGE = 10
 
 DEFAULT_MESSAGES_PER_CLAIM = 10
 
@@ -51,8 +51,7 @@ class Capabilities(enum.IntEnum):
     HIGH_THROUGHPUT = 5
 
 
-@six.add_metaclass(abc.ABCMeta)
-class DriverBase(object):
+class DriverBase(object, metaclass=abc.ABCMeta):
     """Base class for both data and control plane drivers
 
     :param conf: Configuration containing options for this driver.
@@ -77,8 +76,7 @@ class DriverBase(object):
                     pass
 
 
-@six.add_metaclass(abc.ABCMeta)
-class DataDriverBase(DriverBase):
+class DataDriverBase(DriverBase, metaclass=abc.ABCMeta):
     """Interface definition for storage drivers.
 
     Data plane storage drivers are responsible for implementing the
@@ -138,9 +136,9 @@ class DataDriverBase(DriverBase):
         status_template = lambda s, t, r: {'succeeded': s,
                                            'seconds': t,
                                            'ref': r}
-        project = str(uuid.uuid4())
-        queue = str(uuid.uuid4())
-        client = str(uuid.uuid4())
+        project = uuidutils.generate_uuid()
+        queue = uuidutils.generate_uuid()
+        client = uuidutils.generate_uuid()
         msg_template = lambda s: {'ttl': 600, 'body': {'event': 'p_%s' % s}}
         messages = [msg_template(i) for i in range(100)]
         claim_metadata = {'ttl': 60, 'grace': 300}
@@ -154,9 +152,10 @@ class DataDriverBase(DriverBase):
             try:
                 start = time.time()
                 result = callable_operation()
-            except Exception as e:
-                ref = str(uuid.uuid4())
-                LOG.exception(e, extra={'instance_uuid': ref})
+            except Exception:
+                ref = uuidutils.generate_uuid()
+                LOG.exception('Error calling operation.',
+                              extra={'instance_uuid': ref})
                 succeeded = False
             status = status_template(succeeded, time.time() - start, ref)
             op_status[operation_type] = status
@@ -242,9 +241,13 @@ class DataDriverBase(DriverBase):
         """Returns the driver's subscription controller."""
         raise NotImplementedError
 
+    @decorators.lazy_property(write=False)
+    def topic_controller(self):
+        """Returns the driver's topic controller."""
+        return self.control_driver.topic_controller
 
-@six.add_metaclass(abc.ABCMeta)
-class ControlDriverBase(DriverBase):
+
+class ControlDriverBase(DriverBase, metaclass=abc.ABCMeta):
     """Interface definition for control plane storage drivers.
 
     Storage drivers that work at the control plane layer allow one to
@@ -281,6 +284,11 @@ class ControlDriverBase(DriverBase):
         """Returns the driver's queue controller."""
         raise NotImplementedError
 
+    @abc.abstractproperty
+    def topic_controller(self):
+        """Returns the driver's topic controller."""
+        raise NotImplementedError
+
     @abc.abstractmethod
     def close(self):
         """Close connections to the backend."""
@@ -298,8 +306,7 @@ class ControllerBase(object):
         self.driver = driver
 
 
-@six.add_metaclass(abc.ABCMeta)
-class Queue(ControllerBase):
+class Queue(ControllerBase, metaclass=abc.ABCMeta):
     """This class is responsible for managing queues.
 
     Queue operations include CRUD, monitoring, etc.
@@ -309,19 +316,21 @@ class Queue(ControllerBase):
     numbers of queues.
     """
 
-    def list(self, project=None, marker=None,
-             limit=DEFAULT_QUEUES_PER_PAGE, detailed=False):
+    def list(self, project=None, kfilter={}, marker=None,
+             limit=DEFAULT_QUEUES_PER_PAGE, detailed=False, name=None):
         """Base method for listing queues.
 
         :param project: Project id
+        :param kfilter: The key-value of metadata which user want to filter
         :param marker: The last queue name
         :param limit: (Default 10) Max number of queues to return
         :param detailed: Whether metadata is included
+        :param name: The queue name which user want to filter
 
         :returns: An iterator giving a sequence of queues
             and the marker of the next page.
         """
-        return self._list(project, marker, limit, detailed)
+        return self._list(project, kfilter, marker, limit, detailed, name)
 
     _list = abc.abstractmethod(lambda x: None)
 
@@ -332,7 +341,7 @@ class Queue(ControllerBase):
         :param project: Project id
 
         :returns: Dictionary containing queue metadata
-        :raises: DoesNotExist
+        :raises DoesNotExist: if queue metadata does not exist
         """
         return self._get(name, project)
 
@@ -345,7 +354,7 @@ class Queue(ControllerBase):
         :param project: Project id
 
         :returns: Dictionary containing queue metadata
-        :raises: DoesNotExist
+        :raises DoesNotExist: if queue metadata does not exist
         """
         raise NotImplementedError
 
@@ -355,7 +364,7 @@ class Queue(ControllerBase):
         :param name: The queue name
         :param metadata: Queue metadata as a dict
         :param project: Project id
-        :raises: DoesNotExist
+        :raises DoesNotExist: if queue metadata can not be updated
         """
         raise NotImplementedError
 
@@ -405,16 +414,25 @@ class Queue(ControllerBase):
 
     _stats = abc.abstractmethod(lambda x: None)
 
+    def calculate_resource_count(self, project=None):
+        """Base method for calculate queues amount.
 
-@six.add_metaclass(abc.ABCMeta)
-class Message(ControllerBase):
+        :param project: Project id
+        :returns: The number of queues.
+        """
+        return self._calculate_resource_count(project)
+
+    _calculate_resource_count = abc.abstractmethod(lambda x: None)
+
+
+class Message(ControllerBase, metaclass=abc.ABCMeta):
     """This class is responsible for managing message CRUD."""
 
     @abc.abstractmethod
     def list(self, queue, project=None, marker=None,
              limit=DEFAULT_MESSAGES_PER_PAGE,
              echo=False, client_uuid=None,
-             include_claimed=False):
+             include_claimed=False, include_delayed=False):
         """Base method for listing messages.
 
         :param queue: Name of the queue to get the
@@ -428,6 +446,8 @@ class Message(ControllerBase):
         :param client_uuid: A UUID object. Required when echo=False.
         :param include_claimed: omit claimed messages from listing?
         :type include_claimed: bool
+        :param include_delayed: omit delayed messages from listing
+        :type include_delayed: bool
 
         :returns: An iterator giving a sequence of messages and
             the marker of the next page.
@@ -458,7 +478,7 @@ class Message(ControllerBase):
         :param message_id: Message ID
 
         :returns: Dictionary containing message data
-        :raises: DoesNotExist
+        :raises DoesNotExist: if message data can not be got
         """
         raise NotImplementedError
 
@@ -511,7 +531,7 @@ class Message(ControllerBase):
         raise NotImplementedError
 
     @abc.abstractmethod
-    def bulk_delete(self, queue, message_ids, project=None):
+    def bulk_delete(self, queue, message_ids, project=None, claim_ids=None):
         """Base method for deleting multiple messages.
 
         :param queue: Name of the queue to post
@@ -519,6 +539,7 @@ class Message(ControllerBase):
         :param message_ids: A sequence of message IDs
             to be deleted.
         :param project: Project id
+        :param claim_ids: claim IDs passed in by the delete request
         """
         raise NotImplementedError
 
@@ -534,8 +555,7 @@ class Message(ControllerBase):
         raise NotImplementedError
 
 
-@six.add_metaclass(abc.ABCMeta)
-class Claim(ControllerBase):
+class Claim(ControllerBase, metaclass=abc.ABCMeta):
 
     @abc.abstractmethod
     def get(self, queue, claim_id, project=None):
@@ -547,7 +567,7 @@ class Claim(ControllerBase):
         :param project: Project id
 
         :returns: (Claim's metadata, claimed messages)
-        :raises: DoesNotExist
+        :raises DoesNotExist: if claimed messages can not be got
         """
         raise NotImplementedError
 
@@ -593,8 +613,7 @@ class Claim(ControllerBase):
         raise NotImplementedError
 
 
-@six.add_metaclass(abc.ABCMeta)
-class Subscription(ControllerBase):
+class Subscription(ControllerBase, metaclass=abc.ABCMeta):
     """This class is responsible for managing subscriptions of notification.
 
     """
@@ -605,11 +624,11 @@ class Subscription(ControllerBase):
         """Base method for listing subscriptions.
 
         :param queue: Name of the queue to get the subscriptions from.
-        :type queue: six.text_type
+        :type queue: str
         :param project: Project this subscription belongs to.
-        :type project: six.text_type
+        :type project: str
         :param marker: used to determine which subscription to start with
-        :type marker: six.text_type
+        :type marker: str
         :param limit: (Default 10) Max number of results to return
         :type limit: int
         :returns: An iterator giving a sequence of subscriptions
@@ -623,14 +642,14 @@ class Subscription(ControllerBase):
         """Returns a single subscription entry.
 
         :param queue: Name of the queue subscription belongs to.
-        :type queue: six.text_type
+        :type queue: str
         :param subscription_id: ID of this subscription
-        :type subscription_id: six.text_type
+        :type subscription_id: str
         :param project: Project this subscription belongs to.
-        :type project: six.text_type
+        :type project: str
         :returns: Dictionary containing subscription data
         :rtype: {}
-        :raises: SubscriptionDoesNotExist if not found
+        :raises SubscriptionDoesNotExist: if not found
         """
         raise NotImplementedError
 
@@ -639,15 +658,15 @@ class Subscription(ControllerBase):
         """Create a new subscription.
 
         :param queue:The source queue for notifications
-        :type queue: six.text_type
+        :type queue: str
         :param subscriber: The subscriber URI
-        :type subscriber: six.text_type
+        :type subscriber: str
         :param ttl: time to live for this subscription
         :type ttl: int
         :param options: Options used to configure this subscription
         :type options: dict
         :param project: Project id
-        :type project: six.text_type
+        :type project: str
         :returns: True if a subscription was created and False
         if it is failed.
         :rtype: boolean
@@ -659,13 +678,13 @@ class Subscription(ControllerBase):
         """Updates the weight, uris, and/or options of this subscription
 
         :param queue: Name of the queue subscription belongs to.
-        :type queue: six.text_type
+        :type queue: str
         :param name: ID of the subscription
         :type name: text
         :param kwargs: one of: `source`, `subscriber`, `ttl`, `options`
         :type kwargs: dict
-        :raises: SubscriptionDoesNotExist if not found
-        :raises: SubscriptionAlreadyExists on attempt to update in a way to
+        :raises SubscriptionDoesNotExist: if not found
+        :raises SubscriptionAlreadyExists: if attempt to update in a way to
             create duplicate subscription
         """
 
@@ -676,11 +695,11 @@ class Subscription(ControllerBase):
         """Base method for testing subscription existence.
 
         :param queue: Name of the queue subscription belongs to.
-        :type queue: six.text_type
+        :type queue: str
         :param subscription_id: ID of subscription
-        :type subscription_id: six.text_type
+        :type subscription_id: str
         :param project: Project id
-        :type project: six.text_type
+        :type project: str
         :returns: True if a subscription exists and False
             if it does not.
         """
@@ -691,22 +710,52 @@ class Subscription(ControllerBase):
         """Base method for deleting a subscription.
 
         :param queue: Name of the queue subscription belongs to.
-        :type queue: six.text_type
+        :type queue: str
         :param subscription_id: ID of the subscription to be deleted.
-        :type subscription_id: six.text_type
+        :type subscription_id: str
         :param project: Project id
-        :type project: six.text_type
+        :type project: str
+        """
+        raise NotImplementedError
+
+    @abc.abstractmethod
+    def get_with_subscriber(self, queue, subscriber, project=None):
+        """Base method for get a subscription with the subscriber.
+
+        :param queue: Name of the queue subscription belongs to.
+        :type queue: str
+        :param subscriber: link of the subscription to be notified.
+        :type subscriber: str
+        :param project: Project id
+        :type project: str
+        :returns: Dictionary containing subscription data
+        :rtype: dict
+        """
+        raise NotImplementedError
+
+    @abc.abstractmethod
+    def confirm(self, queue, subscription_id, project=None, confirmed=True):
+        """Base method for confirming a subscription.
+
+        :param queue: Name of the queue subscription belongs to.
+        :type queue: str
+        :param subscription_id: ID of the subscription to be deleted.
+        :type subscription_id: str
+        :param project: Project id
+        :type project: str
+        :param confirmed: Confirm a subscription or cancel the confirmation of
+            a subscription.
+        :type confirmed: boolean
         """
         raise NotImplementedError
 
 
-@six.add_metaclass(abc.ABCMeta)
-class PoolsBase(ControllerBase):
+class PoolsBase(ControllerBase, metaclass=abc.ABCMeta):
     """A controller for managing pools."""
 
-    def _check_capabilities(self, uri, group=None, name=None):
+    def _check_capabilities(self, uri, flavor=None, name=None):
         default_store = self.driver.conf.drivers.message_store
-        pool_caps = self.capabilities(group=group, name=name)
+        pool_caps = self.capabilities(flavor=flavor, name=name)
 
         if not pool_caps:
             return True
@@ -714,30 +763,32 @@ class PoolsBase(ControllerBase):
         new_store = utils.load_storage_impl(uri,
                                             default_store=default_store)
 
-        # NOTE(flaper87): Since all pools in a pool group
+        # NOTE(flaper87): Since all pools in a pool flavor
         # are assumed to have the same capabilities, it's
         # fine to check against just 1
         return pool_caps == new_store.BASE_CAPABILITIES
 
-    def capabilities(self, group=None, name=None):
-        """Gets the set of capabilities for this group/name
+    def capabilities(self, flavor=None, name=None):
+        """Gets the set of capabilities for this flavor/name
 
-        :param group: The pool group to get capabilities for
-        :type group: six.text_type
+        :param flavor: The pool flavor to get capabilities for
+        :type flavor: str
         :param name: The pool name to get capabilities for
-        :type name: six.text_type
+        :type name: str
         """
+        pllt = []
         if name:
-            group = list(self._get_pools_by_group(self._get(name)['group']))
+            pool = self.get(name)
+            pllt.append(pool)
         else:
-            group = list(self._get_pools_by_group(group))
+            pllt = list(self._get_pools_by_flavor(flavor))
 
-        if not len(group) > 0:
+        if not len(pllt) > 0:
             return ()
 
         default_store = self.driver.conf.drivers.message_store
 
-        pool_store = utils.load_storage_impl(group[0]['uri'],
+        pool_store = utils.load_storage_impl(pllt[0]['uri'],
                                              default_store=default_store)
 
         return pool_store.BASE_CAPABILITIES
@@ -747,7 +798,7 @@ class PoolsBase(ControllerBase):
         """Lists all registered pools.
 
         :param marker: used to determine which pool to start with
-        :type marker: six.text_type
+        :type marker: str
         :param limit: (Default 10) Max number of results to return
         :type limit: int
         :param detailed: whether to include options
@@ -760,54 +811,57 @@ class PoolsBase(ControllerBase):
 
     _list = abc.abstractmethod(lambda x: None)
 
-    def create(self, name, weight, uri, group=None, options=None):
+    def create(self, name, weight, uri, flavor=None, options=None):
         """Registers a pool entry.
 
         :param name: The name of this pool
-        :type name: six.text_type
+        :type name: str
         :param weight: the likelihood that this pool will be used
         :type weight: int
         :param uri: A URI that can be used by a storage client
             (e.g., pymongo) to access this pool.
-        :type uri: six.text_type
-        :param group: The group of this pool
-        :type group: six.text_type
+        :type uri: str
+        :param flavor: The flavor of this pool
+        :type flavor: str
         :param options: Options used to configure this pool
         :type options: dict
         """
-        if not self._check_capabilities(uri, group=group):
+        flavor_obj = {}
+        if flavor is not None:
+            flavor_obj["name"] = flavor
+        if not self._check_capabilities(uri, flavor=flavor_obj):
             raise errors.PoolCapabilitiesMismatch()
 
-        return self._create(name, weight, uri, group, options)
+        return self._create(name, weight, uri, flavor, options)
 
     _create = abc.abstractmethod(lambda x: None)
 
-    def get_pools_by_group(self, group=None, detailed=False):
-        """Returns a pool list filtered by given pool group.
+    def get_pools_by_flavor(self, flavor=None, detailed=False):
+        """Returns a pool list filtered by given pool flavor.
 
-        :param group: The group to filter on. `None` returns
-            pools that are not assigned to any pool group.
-        :type group: six.text_type
+        :param flavor: The flavor to filter on. `None` returns
+            pools that are not assigned to any pool flavor.
+        :type flavor: str
         :param detailed: Should the options data be included?
         :type detailed: bool
         :returns: weight, uri, and options for this pool
         :rtype: {}
-        :raises: PoolDoesNotExist if not found
+        :raises PoolDoesNotExist: if not found
         """
-        return self._get_pools_by_group(group, detailed)
+        return self._get_pools_by_flavor(flavor, detailed)
 
-    _get_pools_by_group = abc.abstractmethod(lambda x: None)
+    _get_pools_by_flavor = abc.abstractmethod(lambda x: None)
 
     def get(self, name, detailed=False):
         """Returns a single pool entry.
 
         :param name: The name of this pool
-        :type name: six.text_type
+        :type name: str
         :param detailed: Should the options data be included?
         :type detailed: bool
         :returns: weight, uri, and options for this pool
         :rtype: {}
-        :raises: PoolDoesNotExist if not found
+        :raises PoolDoesNotExist: if not found
         """
         return self._get(name, detailed)
 
@@ -817,7 +871,7 @@ class PoolsBase(ControllerBase):
         """Returns a single pool entry.
 
         :param name: The name of this pool
-        :type name: six.text_type
+        :type name: str
         :returns: True if the pool exists
         :rtype: bool
         """
@@ -829,7 +883,7 @@ class PoolsBase(ControllerBase):
         """Removes a pool entry.
 
         :param name: The name of this pool
-        :type name: six.text_type
+        :type name: str
         :rtype: None
         """
         return self._delete(name)
@@ -843,7 +897,7 @@ class PoolsBase(ControllerBase):
         :type name: text
         :param kwargs: one of: `uri`, `weight`, `options`
         :type kwargs: dict
-        :raises: PoolDoesNotExist
+        :raises PoolDoesNotExist: if not found
         """
         uri = kwargs.get('uri')
         if uri and not self._check_capabilities(uri, name=name):
@@ -860,8 +914,7 @@ class PoolsBase(ControllerBase):
     _drop_all = abc.abstractmethod(lambda x: None)
 
 
-@six.add_metaclass(abc.ABCMeta)
-class CatalogueBase(ControllerBase):
+class CatalogueBase(ControllerBase, metaclass=abc.ABCMeta):
     """A controller for managing the catalogue.
 
     The catalogue is responsible for maintaining a mapping
@@ -874,7 +927,7 @@ class CatalogueBase(ControllerBase):
 
         :param project: The project to use when filtering through queue
                         entries.
-        :type project: six.text_type
+        :type project: str
         :returns: [{'project': ..., 'queue': ..., 'pool': ...},]
         :rtype: [dict]
         """
@@ -886,12 +939,12 @@ class CatalogueBase(ControllerBase):
         """Returns the pool identifier for the given queue.
 
         :param project: Namespace to search for the given queue
-        :type project: six.text_type
+        :type project: str
         :param queue: The name of the queue to search for
-        :type queue: six.text_type
+        :type queue: str
         :returns: {'pool': ...}
         :rtype: dict
-        :raises: QueueNotMapped
+        :raises QueueNotMapped: if queue is not mapped
         """
 
         raise NotImplementedError
@@ -901,9 +954,9 @@ class CatalogueBase(ControllerBase):
         """Determines whether the given queue exists under project.
 
         :param project: Namespace to check.
-        :type project: six.text_type
+        :type project: str
         :param queue: str - Particular queue to check for
-        :type queue: six.text_type
+        :type queue: str
         :return: True if the queue exists under this project
         :rtype: bool
         """
@@ -913,11 +966,11 @@ class CatalogueBase(ControllerBase):
         """Creates a new catalogue entry, or updates it if it already exists.
 
         :param project: str - Namespace to insert the given queue into
-        :type project: six.text_type
+        :type project: str
         :param queue: str - The name of the queue to insert
-        :type queue: six.text_type
+        :type queue: str
         :param pool: pool identifier to associate this queue with
-        :type pool: six.text_type
+        :type pool: str
         """
 
         raise NotImplementedError
@@ -927,9 +980,9 @@ class CatalogueBase(ControllerBase):
         """Removes this entry from the catalogue.
 
         :param project: The namespace to search for this queue
-        :type project: six.text_type
+        :type project: str
         :param queue: The queue name to remove
-        :type queue: six.text_type
+        :type queue: str
         """
 
         raise NotImplementedError
@@ -939,12 +992,12 @@ class CatalogueBase(ControllerBase):
         """Updates the pool identifier for this queue.
 
         :param project: Namespace to search
-        :type project: six.text_type
+        :type project: str
         :param queue: The name of the queue
-        :type queue: six.text_type
+        :type queue: str
         :param pools: The name of the pool where this project/queue lives.
-        :type pools: six.text_type
-        :raises: QueueNotMapped
+        :type pools: str
+        :raises QueueNotMapped: if queue is not mapped
         """
 
         raise NotImplementedError
@@ -956,8 +1009,7 @@ class CatalogueBase(ControllerBase):
         raise NotImplementedError
 
 
-@six.add_metaclass(abc.ABCMeta)
-class FlavorsBase(ControllerBase):
+class FlavorsBase(ControllerBase, metaclass=abc.ABCMeta):
     """A controller for managing flavors."""
 
     @abc.abstractmethod
@@ -965,9 +1017,9 @@ class FlavorsBase(ControllerBase):
         """Lists all registered flavors.
 
         :param project: Project this flavor belongs to.
-        :type project: six.text_type
+        :type project: str
         :param marker: used to determine which flavor to start with
-        :type marker: six.text_type
+        :type marker: str
         :param limit: (Default 10) Max number of results to return
         :type limit: int
         :returns: A list of flavors - name, project, flavor
@@ -977,15 +1029,15 @@ class FlavorsBase(ControllerBase):
         raise NotImplementedError
 
     @abc.abstractmethod
-    def create(self, name, pool, project=None, capabilities=None):
+    def create(self, name, project=None, capabilities=None):
         """Registers a flavor entry.
 
         :param name: The name of this flavor
-        :type name: six.text_type
+        :type name: str
         :param project: Project this flavor belongs to.
-        :type project: six.text_type
+        :type project: str
         :param pool: The name of the pool to use for this flavor.
-        :type pool: six.text_type
+        :type pool: str
         :param capabilities: Flavor capabilities
         :type capabilities: dict
         """
@@ -997,11 +1049,11 @@ class FlavorsBase(ControllerBase):
         """Returns a single flavor entry.
 
         :param name: The name of this flavor
-        :type name: six.text_type
+        :type name: str
         :param project: Project this flavor belongs to.
-        :type project: six.text_type
+        :type project: str
         :rtype: {}
-        :raises: FlavorDoesNotExist if not found
+        :raises FlavorDoesNotExist: if not found
         """
 
         raise NotImplementedError
@@ -1011,9 +1063,9 @@ class FlavorsBase(ControllerBase):
         """Verifies whether the flavor exists.
 
         :param name: The name of this flavor
-        :type name: six.text_type
+        :type name: str
         :param project: Project this flavor belongs to.
-        :type project: six.text_type
+        :type project: str
         :returns: True if the flavor exists
         :rtype: bool
         """
@@ -1025,9 +1077,9 @@ class FlavorsBase(ControllerBase):
         """Removes a flavor entry.
 
         :param name: The name of this flavor
-        :type name: six.text_type
+        :type name: str
         :param project: Project this flavor belongs to.
-        :type project: six.text_type
+        :type project: str
         :rtype: None
         """
 
@@ -1040,10 +1092,10 @@ class FlavorsBase(ControllerBase):
         :param name: Name of the flavor
         :type name: text
         :param project: Project this flavor belongs to.
-        :type project: six.text_type
+        :type project: str
         :param kwargs: one of: `uri`, `weight`, `options`
         :type kwargs: dict
-        :raises: FlavorDoesNotExist
+        :raises FlavorDoesNotExist: if not found
         """
 
         raise NotImplementedError
@@ -1053,3 +1105,112 @@ class FlavorsBase(ControllerBase):
         """Deletes all flavors from storage."""
 
         raise NotImplementedError
+
+
+class Topic(ControllerBase, metaclass=abc.ABCMeta):
+    """This class is responsible for managing topics.
+
+    Topic operations include CRUD, etc.
+
+    Storage driver implementations of this class should
+    be capable of handling high workloads and huge
+    numbers of topics.
+    """
+
+    def list(self, project=None, kfilter={}, marker=None,
+             limit=DEFAULT_TOPICS_PER_PAGE, detailed=False, name=None):
+        """Base method for listing topics.
+
+        :param project: Project id
+        :param kfilter: The key-value of metadata which user want to filter
+        :param marker: The last topic name
+        :param limit: (Default 10) Max number of topics to return
+        :param detailed: Whether metadata is included
+        :param name: The topic name which user want to filter
+
+        :returns: An iterator giving a sequence of topics
+            and the marker of the next page.
+        """
+        return self._list(project, kfilter, marker, limit, detailed, name)
+
+    _list = abc.abstractmethod(lambda x: None)
+
+    def get(self, name, project=None):
+        """Base method for topic metadata retrieval.
+
+        :param name: The topic name
+        :param project: Project id
+
+        :returns: Dictionary containing topic metadata
+        :raises DoesNotExist: if topic metadata does not exist
+        """
+        return self._get(name, project)
+
+    _get = abc.abstractmethod(lambda x: None)
+
+    def get_metadata(self, name, project=None):
+        """Base method for topic metadata retrieval.
+
+        :param name: The topic name
+        :param project: Project id
+
+        :returns: Dictionary containing topic metadata
+        :raises DoesNotExist: if topic metadata does not exist
+        """
+        raise NotImplementedError
+
+    def set_metadata(self, name, metadata, project=None):
+        """Base method for updating a topic metadata.
+
+        :param name: The topic name
+        :param metadata: Topic metadata as a dict
+        :param project: Project id
+        :raises DoesNotExist: if topic metadata can not be updated
+        """
+        raise NotImplementedError
+
+    def create(self, name, metadata=None, project=None):
+        """Base method for topic creation.
+
+        :param name: The topic name
+        :param project: Project id
+        :returns: True if a topic was created and False
+            if it was updated.
+        """
+        return self._create(name, metadata, project)
+
+    _create = abc.abstractmethod(lambda x: None)
+
+    def exists(self, name, project=None):
+        """Base method for testing topic existence.
+
+        :param name: The topic name
+        :param project: Project id
+        :returns: True if a topic exists and False
+            if it does not.
+        """
+        return self._exists(name, project)
+
+    _exists = abc.abstractmethod(lambda x: None)
+
+    def delete(self, name, project=None):
+        """Base method for deleting a topic.
+
+        :param name: The topic name
+        :param project: Project id
+        """
+        return self._delete(name, project)
+
+    _delete = abc.abstractmethod(lambda x: None)
+
+    def stats(self, name, project=None):
+        """Base method for topic stats.
+
+        :param name: The topic name
+        :param project: Project id
+        :returns: Dictionary with the
+            queue stats
+        """
+        return self._stats(name, project)
+
+    _stats = abc.abstractmethod(lambda x: None)
