@@ -142,10 +142,11 @@ class DataDriver(storage.DataDriverBase):
         message_volume = {'free': 0, 'claimed': 0, 'total': 0}
 
         for msg_col in [db.messages for db in self.message_databases]:
-            msg_count_claimed = msg_col.find({'c.id': {'$ne': None}}).count()
+            msg_count_claimed = msg_col.count_documents({'c.id':
+                                                        {'$ne': None}})
             message_volume['claimed'] += msg_count_claimed
 
-            msg_count_total = msg_col.find().count()
+            msg_count_total = msg_col.count_documents({})
             message_volume['total'] += msg_count_total
 
         message_volume['free'] = (message_volume['total'] -
@@ -271,6 +272,17 @@ class ControlDriver(storage.ControlDriverBase):
         name = self.mongodb_conf.database + '_queues'
         return self.connection[name]
 
+    @decorators.lazy_property(write=False)
+    def topics_database(self):
+        """Database dedicated to the "topics" collection.
+
+        The topics collection is separated out into its own database
+        to avoid writer lock contention with the messages collections.
+        """
+
+        name = self.mongodb_conf.database + '_topics'
+        return self.connection[name]
+
     @decorators.lazy_property(write=False)
     def queue_controller(self):
         controller = controllers.QueueController(self)
@@ -308,3 +320,13 @@ class ControlDriver(storage.ControlDriverBase):
             return profiler.trace_cls("mongodb_flavors_controller")(controller)
         else:
             return controller
+
+    @decorators.lazy_property(write=False)
+    def topic_controller(self):
+        controller = controllers.TopicController(self)
+        if (self.conf.profiler.enabled and
+                (self.conf.profiler.trace_message_store or
+                    self.conf.profiler.trace_management_store)):
+            return profiler.trace_cls("mongodb_topics_controller")(controller)
+        else:
+            return controller
