@@ -23,7 +23,9 @@ Field Mappings:
 
 import datetime
 import time
+import uuid
 
+from bson import binary
 from bson import objectid
 from oslo_log import log as logging
 from oslo_utils import timeutils
@@ -34,6 +36,7 @@ from zaqar.i18n import _
 from zaqar import storage
 from zaqar.storage import errors
 from zaqar.storage.mongodb import utils
+from zaqar.storage import utils as s_utils
 
 
 LOG = logging.getLogger(__name__)
@@ -135,6 +138,8 @@ class MessageController(storage.Message):
             claim            ->     c
             client uuid      ->     u
             transaction      ->    tx
+            delay            ->     d
+            checksum         ->    cs
     """
 
     def __init__(self, *args, **kwargs):
@@ -166,28 +171,28 @@ class MessageController(storage.Message):
     def _ensure_indexes(self, collection):
         """Ensures that all indexes are created."""
 
-        collection.ensure_index(TTL_INDEX_FIELDS,
+        collection.create_index(TTL_INDEX_FIELDS,
                                 name='ttl',
                                 expireAfterSeconds=0,
                                 background=True)
 
-        collection.ensure_index(ACTIVE_INDEX_FIELDS,
+        collection.create_index(ACTIVE_INDEX_FIELDS,
                                 name='active',
                                 background=True)
 
-        collection.ensure_index(CLAIMED_INDEX_FIELDS,
+        collection.create_index(CLAIMED_INDEX_FIELDS,
                                 name='claimed',
                                 background=True)
 
-        collection.ensure_index(COUNTING_INDEX_FIELDS,
+        collection.create_index(COUNTING_INDEX_FIELDS,
                                 name='counting',
                                 background=True)
 
-        collection.ensure_index(MARKER_INDEX_FIELDS,
+        collection.create_index(MARKER_INDEX_FIELDS,
                                 name='queue_marker',
                                 background=True)
 
-        collection.ensure_index(TRANSACTION_INDEX_FIELDS,
+        collection.create_index(TRANSACTION_INDEX_FIELDS,
                                 name='transaction',
                                 background=True)
 
@@ -226,11 +231,12 @@ class MessageController(storage.Message):
         """
         scope = utils.scope_queue_name(queue_name, project)
         collection = self._collection(queue_name, project)
-        collection.remove({PROJ_QUEUE: scope}, w=0)
+        collection.delete_many({PROJ_QUEUE: scope})
 
     def _list(self, queue_name, project=None, marker=None,
               echo=False, client_uuid=None, projection=None,
-              include_claimed=False, sort=1, limit=None):
+              include_claimed=False, include_delayed=False,
+              sort=1, limit=None, count=False):
         """Message document listing helper.
 
         :param queue_name: Name of the queue to list
@@ -243,10 +249,13 @@ class MessageController(storage.Message):
             client_uuid
         :param client_uuid: (Default None) UUID for the client that
             originated this request
-        :param fields: (Default None) Fields to include in emitted
-            documents
+        :param projection: (Default None) a list of field names that should be
+            returned in the result set or a dict specifying the fields to
+            include or exclude
         :param include_claimed: (Default False) Whether to include
             claimed messages, not just active ones
+        :param include_delayed: (Default False) Whether to include
+            delayed messages, not just active ones
         :param sort: (Default 1) Sort order for the listing. Pass 1 for
             ascending (oldest message first), or -1 for descending (newest
             message first).
@@ -254,6 +263,7 @@ class MessageController(storage.Message):
             to list. The results may include fewer messages than the
             requested `limit` if not enough are available. If limit is
             not specified
+        :param count: (Default False) If return the collection's count
 
         :returns: Generator yielding up to `limit` messages.
         """
@@ -277,6 +287,12 @@ class MessageController(storage.Message):
         }
 
         if not echo:
+            if (client_uuid is not None) and not isinstance(client_uuid,
+                                                            uuid.UUID):
+                client_uuid = uuid.UUID(client_uuid)
+                client_uuid = binary.Binary.from_uuid(client_uuid)
+            elif isinstance(client_uuid, uuid.UUID):
+                client_uuid = binary.Binary.from_uuid(client_uuid)
             query['u'] = {'$ne': client_uuid}
 
         if marker is not None:
@@ -289,16 +305,31 @@ class MessageController(storage.Message):
             # any claim, or are part of an expired claim.
             query['c.e'] = {'$lte': now}
 
+        if not include_delayed:
+            # NOTE(cdyangzhenyu): Only include messages that are not
+            # part of any delay, or are part of an expired delay. if
+            # the message has no attribute 'd', it will also be obtained.
+            # This is for compatibility with old data.
+            query['$or'] = [{'d': {'$lte': now}},
+                            {'d': {'$exists': False}}]
+
         # Construct the request
         cursor = collection.find(query,
                                  projection=projection,
                                  sort=[('k', sort)])
+        ntotal = None
+        if count:
+            ntotal = collection.count_documents(query)
 
         if limit is not None:
             cursor.limit(limit)
+            if count:
+                ntotal = collection.count_documents(query, limit=limit)
 
         # NOTE(flaper87): Suggest the index to use for this query to
         # ensure the most performant one is chosen.
+        if count:
+            return cursor.hint(ACTIVE_INDEX_FIELDS), ntotal
         return cursor.hint(ACTIVE_INDEX_FIELDS)
 
     # ----------------------------------------------------------------------
@@ -333,16 +364,17 @@ class MessageController(storage.Message):
             query['c.e'] = {'$lte': timeutils.utcnow_ts()}
 
         collection = self._collection(queue_name, project)
-        return collection.count(filter=query, hint=COUNTING_INDEX_FIELDS)
+        return collection.count_documents(filter=query,
+                                          hint=COUNTING_INDEX_FIELDS)
 
     def _active(self, queue_name, marker=None, echo=False,
                 client_uuid=None, projection=None, project=None,
-                limit=None):
+                limit=None, include_delayed=False):
 
         return self._list(queue_name, project=project, marker=marker,
                           echo=echo, client_uuid=client_uuid,
                           projection=projection, include_claimed=False,
-                          limit=limit)
+                          include_delayed=include_delayed, limit=limit)
 
     def _claimed(self, queue_name, claim_id,
                  expires=None, limit=None, project=None):
@@ -368,8 +400,10 @@ class MessageController(storage.Message):
         msgs = collection.find(query, sort=[('k', 1)], **kwargs).hint(
             CLAIMED_INDEX_FIELDS)
 
+        ntotal = collection.count_documents(query)
         if limit is not None:
             msgs = msgs.limit(limit)
+            ntotal = collection.count_documents(query, limit=limit)
 
         now = timeutils.utcnow_ts()
 
@@ -379,7 +413,7 @@ class MessageController(storage.Message):
 
             return doc
 
-        return utils.HookedCursor(msgs, denormalizer)
+        return utils.HookedCursor(msgs, denormalizer, ntotal=ntotal)
 
     def _unclaim(self, queue_name, claim_id, project=None):
         cid = utils.to_oid(claim_id)
@@ -395,9 +429,9 @@ class MessageController(storage.Message):
         scope = utils.scope_queue_name(queue_name, project)
         collection = self._collection(queue_name, project)
 
-        collection.update({PROJ_QUEUE: scope, 'c.id': cid},
-                          {'$set': {'c': {'id': None, 'e': now}}},
-                          upsert=False, multi=True)
+        collection.update_many({PROJ_QUEUE: scope, 'c.id': cid},
+                               {'$set': {'c': {'id': None, 'e': now}}},
+                               upsert=False)
 
     def _inc_counter(self, queue_name, project=None, amount=1, window=None):
         """Increments the message counter and returns the new value.
@@ -413,7 +447,7 @@ class MessageController(storage.Message):
             was specified, and the counter has already been updated
             within the specified time period.
 
-        :raises: storage.errors.QueueDoesNotExist
+        :raises QueueDoesNotExist: if not found
         """
 
         # NOTE(flaper87): If this `if` is True, it means we're
@@ -442,17 +476,17 @@ class MessageController(storage.Message):
                     projection={'c.v': 1, '_id': 0})
 
                 break
-            except pymongo.errors.AutoReconnect as ex:
-                LOG.exception(ex)
+            except pymongo.errors.AutoReconnect:
+                LOG.exception('Auto reconnect error')
 
         if doc is None:
             if window is None:
                 # NOTE(kgriffs): Since we did not filter by a time window,
                 # the queue should have been found and updated. Perhaps
                 # the queue has been deleted?
-                message = _(u'Failed to increment the message '
-                            u'counter for queue %(name)s and '
-                            u'project %(project)s')
+                message = (u'Failed to increment the message '
+                           u'counter for queue %(name)s and '
+                           u'project %(project)s')
                 message %= dict(name=queue_name, project=project)
 
                 LOG.warning(message)
@@ -507,8 +541,8 @@ class MessageController(storage.Message):
                 projection={'c.v': 1, '_id': 0})
 
             return doc['c']['v']
-        except pymongo.errors.AutoReconnect as ex:
-            LOG.exception(ex)
+        except pymongo.errors.AutoReconnect:
+            LOG.exception('Auto reconnect error')
 
     # ----------------------------------------------------------------------
     # Public interface
@@ -516,7 +550,8 @@ class MessageController(storage.Message):
 
     def list(self, queue_name, project=None, marker=None,
              limit=storage.DEFAULT_MESSAGES_PER_PAGE,
-             echo=False, client_uuid=None, include_claimed=False):
+             echo=False, client_uuid=None, include_claimed=False,
+             include_delayed=False):
 
         if marker is not None:
             try:
@@ -524,9 +559,13 @@ class MessageController(storage.Message):
             except ValueError:
                 yield iter([])
 
-        messages = self._list(queue_name, project=project, marker=marker,
-                              client_uuid=client_uuid, echo=echo,
-                              include_claimed=include_claimed, limit=limit)
+        messages, ntotal = self._list(queue_name, project=project,
+                                      marker=marker,
+                                      client_uuid=client_uuid, echo=echo,
+                                      include_claimed=include_claimed,
+                                      include_delayed=include_delayed,
+                                      limit=limit,
+                                      count=True)
 
         marker_id = {}
 
@@ -539,7 +578,7 @@ class MessageController(storage.Message):
 
             return _basic_message(msg, now)
 
-        yield utils.HookedCursor(messages, denormalizer)
+        yield utils.HookedCursor(messages, denormalizer, ntotal=ntotal)
         yield str(marker_id['next'])
 
     @utils.raises_conn_error
@@ -600,11 +639,12 @@ class MessageController(storage.Message):
         # NOTE(flaper87): Should this query
         # be sorted?
         messages = collection.find(query).hint(ID_INDEX_FIELDS)
+        ntotal = collection.count_documents(query)
 
         def denormalizer(msg):
             return _basic_message(msg, now)
 
-        return utils.HookedCursor(messages, denormalizer)
+        return utils.HookedCursor(messages, denormalizer, ntotal=ntotal)
 
     @utils.raises_conn_error
     @utils.retries_on_autoreconnect
@@ -617,6 +657,13 @@ class MessageController(storage.Message):
         if not self._queue_ctrl.exists(queue_name, project):
             raise errors.QueueDoesNotExist(queue_name, project)
 
+        if (client_uuid is not None) and not isinstance(client_uuid,
+                                                        uuid.UUID):
+            client_uuid = uuid.UUID(client_uuid)
+            client_uuid = binary.Binary.from_uuid(client_uuid)
+        elif isinstance(client_uuid, uuid.UUID):
+            client_uuid = binary.Binary.from_uuid(client_uuid)
+
         # NOTE(flaper87): Make sure the counter exists. This method
         # is an upsert.
         self._get_counter(queue_name, project)
@@ -630,24 +677,28 @@ class MessageController(storage.Message):
                                         project,
                                         amount=msgs_n) - msgs_n
 
-        prepared_messages = [
-            {
+        prepared_messages = []
+        for index, message in enumerate(messages):
+            msg = {
                 PROJ_QUEUE: utils.scope_queue_name(queue_name, project),
                 't': message['ttl'],
                 'e': now_dt + datetime.timedelta(seconds=message['ttl']),
                 'u': client_uuid,
-                'c': {'id': None, 'e': now},
+                'c': {'id': None, 'e': now, 'c': 0},
+                'd': now + message.get('delay', 0),
                 'b': message['body'] if 'body' in message else {},
                 'k': next_marker + index,
-                'tx': None,
-            }
+                'tx': None
+                }
+            if self.driver.conf.enable_checksum:
+                msg['cs'] = s_utils.get_checksum(message.get('body', None))
 
-            for index, message in enumerate(messages)
-        ]
+            prepared_messages.append(msg)
 
-        ids = collection.insert(prepared_messages, check_keys=False)
+        res = collection.insert_many(prepared_messages,
+                                     bypass_document_validation=True)
 
-        return [str(id_) for id_ in ids]
+        return [str(id_) for id_ in res.inserted_ids]
 
     @utils.raises_conn_error
     @utils.retries_on_autoreconnect
@@ -698,19 +749,31 @@ class MessageController(storage.Message):
 
                     raise errors.MessageNotClaimed(message_id)
 
-        collection.remove(query['_id'], w=0)
+        collection.delete_one(query)
 
     @utils.raises_conn_error
     @utils.retries_on_autoreconnect
-    def bulk_delete(self, queue_name, message_ids, project=None):
+    def bulk_delete(self, queue_name, message_ids, project=None,
+                    claim_ids=None):
         message_ids = [mid for mid in map(utils.to_oid, message_ids) if mid]
+        if claim_ids:
+            claim_ids = [cid for cid in map(utils.to_oid, claim_ids) if cid]
         query = {
             '_id': {'$in': message_ids},
             PROJ_QUEUE: utils.scope_queue_name(queue_name, project),
         }
 
         collection = self._collection(queue_name, project)
-        collection.remove(query, w=0)
+        if claim_ids:
+            message_claim_ids = []
+            messages = collection.find(query).hint(ID_INDEX_FIELDS)
+            for message in messages:
+                message_claim_ids.append(message['c']['id'])
+            for cid in claim_ids:
+                if cid not in message_claim_ids:
+                    raise errors.ClaimDoesNotExist(cid, queue_name, project)
+
+        collection.delete_many(query)
 
     @utils.raises_conn_error
     @utils.retries_on_autoreconnect
@@ -727,9 +790,8 @@ class MessageController(storage.Message):
         collection = self._collection(queue_name, project)
         projection = {'_id': 1, 't': 1, 'b': 1, 'c.id': 1}
 
-        messages = (collection.find_and_modify(query,
-                                               projection=projection,
-                                               remove=True)
+        messages = (collection.find_one_and_delete(query,
+                                                   projection=projection)
                     for _ in range(limit))
 
         final_messages = [_basic_message(message, now)
@@ -744,20 +806,20 @@ class FIFOMessageController(MessageController):
     def _ensure_indexes(self, collection):
         """Ensures that all indexes are created."""
 
-        collection.ensure_index(TTL_INDEX_FIELDS,
+        collection.create_index(TTL_INDEX_FIELDS,
                                 name='ttl',
                                 expireAfterSeconds=0,
                                 background=True)
 
-        collection.ensure_index(ACTIVE_INDEX_FIELDS,
+        collection.create_index(ACTIVE_INDEX_FIELDS,
                                 name='active',
                                 background=True)
 
-        collection.ensure_index(CLAIMED_INDEX_FIELDS,
+        collection.create_index(CLAIMED_INDEX_FIELDS,
                                 name='claimed',
                                 background=True)
 
-        collection.ensure_index(COUNTING_INDEX_FIELDS,
+        collection.create_index(COUNTING_INDEX_FIELDS,
                                 name='counting',
                                 background=True)
 
@@ -768,12 +830,12 @@ class FIFOMessageController(MessageController):
         # to miss a message when there is more than one
         # producer posting messages to the same queue, in
         # parallel.
-        collection.ensure_index(MARKER_INDEX_FIELDS,
+        collection.create_index(MARKER_INDEX_FIELDS,
                                 name='queue_marker',
                                 unique=True,
                                 background=True)
 
-        collection.ensure_index(TRANSACTION_INDEX_FIELDS,
+        collection.create_index(TRANSACTION_INDEX_FIELDS,
                                 name='transaction',
                                 background=True)
 
@@ -808,20 +870,30 @@ class FIFOMessageController(MessageController):
         # Unique transaction ID to facilitate atomic batch inserts
         transaction = objectid.ObjectId()
 
-        prepared_messages = [
-            {
+        if (client_uuid is not None) and not isinstance(client_uuid,
+                                                        uuid.UUID):
+            client_uuid = uuid.UUID(client_uuid)
+            client_uuid = binary.Binary.from_uuid(client_uuid)
+        elif isinstance(client_uuid, uuid.UUID):
+            client_uuid = binary.Binary.from_uuid(client_uuid)
+
+        prepared_messages = []
+        for index, message in enumerate(messages):
+            msg = {
                 PROJ_QUEUE: utils.scope_queue_name(queue_name, project),
                 't': message['ttl'],
                 'e': now_dt + datetime.timedelta(seconds=message['ttl']),
                 'u': client_uuid,
-                'c': {'id': None, 'e': now},
+                'c': {'id': None, 'e': now, 'c': 0},
+                'd': now + message.get('delay', 0),
                 'b': message['body'] if 'body' in message else {},
                 'k': next_marker + index,
-                'tx': transaction,
-            }
+                'tx': None
+                }
+            if self.driver.conf.enable_checksum:
+                msg['cs'] = s_utils.get_checksum(message.get('body', None))
 
-            for index, message in enumerate(messages)
-        ]
+            prepared_messages.append(msg)
 
         # NOTE(kgriffs): Don't take the time to do a 2-phase insert
         # if there is no way for it to partially succeed.
@@ -838,7 +910,8 @@ class FIFOMessageController(MessageController):
         # before the operation is abandoned is 49.95 seconds.
         for attempt in self._retry_range:
             try:
-                ids = collection.insert(prepared_messages, check_keys=False)
+                res = collection.insert_many(prepared_messages,
+                                             bypass_document_validation=True)
 
                 # Log a message if we retried, for debugging perf issues
                 if attempt != 0:
@@ -849,7 +922,7 @@ class FIFOMessageController(MessageController):
                     LOG.debug(msgtmpl,
                               dict(queue=queue_name,
                                    attempts=attempt + 1,
-                                   num_messages=len(ids),
+                                   num_messages=len(res.inserted_ids),
                                    project=project))
 
                 # Update the counter in preparation for the next batch
@@ -860,20 +933,22 @@ class FIFOMessageController(MessageController):
                 # such that the competing marker's will start at a
                 # unique number, 1 past the max of the messages just
                 # inserted above.
-                self._inc_counter(queue_name, project, amount=len(ids))
+                self._inc_counter(queue_name, project,
+                                  amount=len(res.inserted_ids))
 
                 # NOTE(kgriffs): Finalize the insert once we can say that
                 # all the messages made it. This makes bulk inserts
                 # atomic, assuming queries filter out any non-finalized
                 # messages.
                 if transaction is not None:
-                    collection.update({'tx': transaction},
-                                      {'$set': {'tx': None}},
-                                      upsert=False, multi=True)
+                    collection.update_many({'tx': transaction},
+                                           {'$set': {'tx': None}},
+                                           upsert=False)
 
-                return [str(id_) for id_ in ids]
+                return [str(id_) for id_ in res.inserted_ids]
 
-            except pymongo.errors.DuplicateKeyError as ex:
+            except (pymongo.errors.DuplicateKeyError,
+                    pymongo.errors.BulkWriteError):
                 # TODO(kgriffs): Record stats of how often retries happen,
                 # and how many attempts, on average, are required to insert
                 # messages.
@@ -899,8 +974,8 @@ class FIFOMessageController(MessageController):
                 # this situation can not happen.
                 elapsed = timeutils.utcnow_ts() - now
                 if elapsed > MAX_RETRY_POST_DURATION:
-                    msgtmpl = _(u'Exceeded maximum retry duration for queue '
-                                u'"%(queue)s" under project %(project)s')
+                    msgtmpl = (u'Exceeded maximum retry duration for queue '
+                               u'"%(queue)s" under project %(project)s')
 
                     LOG.warning(msgtmpl,
                                 dict(queue=queue_name, project=project))
@@ -943,8 +1018,9 @@ class FIFOMessageController(MessageController):
                     next_marker = self._get_counter(
                         queue_name, project)
                 else:
-                    msgtmpl = (u'Detected a stalled message counter for '
-                               u'queue "%(queue)s" under project %(project)s. '
+                    msgtmpl = (u'Detected a stalled message counter '
+                               u'for queue "%(queue)s" under '
+                               u'project %(project)s.'
                                u'The counter was incremented to %(value)d.')
 
                     LOG.warning(msgtmpl,
@@ -954,13 +1030,12 @@ class FIFOMessageController(MessageController):
 
                 for index, message in enumerate(prepared_messages):
                     message['k'] = next_marker + index
-
-            except Exception as ex:
-                LOG.exception(ex)
+            except Exception:
+                LOG.exception('Error parsing document')
                 raise
 
-        msgtmpl = _(u'Hit maximum number of attempts (%(max)s) for queue '
-                    u'"%(queue)s" under project %(project)s')
+        msgtmpl = (u'Hit maximum number of attempts (%(max)s) for queue '
+                   u'"%(queue)s" under project %(project)s')
 
         LOG.warning(msgtmpl,
                     dict(max=self.driver.mongodb_conf.max_attempts,
@@ -978,14 +1053,18 @@ def _is_claimed(msg, now):
 def _basic_message(msg, now):
     oid = msg['_id']
     age = now - utils.oid_ts(oid)
-
-    return {
+    res = {
         'id': str(oid),
         'age': int(age),
         'ttl': msg['t'],
+        'claim_count': msg['c'].get('c', 0),
         'body': msg['b'],
         'claim_id': str(msg['c']['id']) if msg['c']['id'] else None
-    }
+        }
+    if msg.get('cs'):
+        res['checksum'] = msg.get('cs')
+
+    return res
 
 
 class MessageQueueHandler(object):
