@@ -14,14 +14,13 @@
 # limitations under the License.
 
 import datetime
-import uuid
+from unittest import mock
 
 import ddt
 import falcon
-import mock
 from oslo_serialization import jsonutils
 from oslo_utils import timeutils
-import six
+from oslo_utils import uuidutils
 from testtools import matchers
 
 from zaqar import tests as testing
@@ -64,7 +63,7 @@ class TestMessagesMongoDB(base.V1Base):
         self.simulate_put(self.queue_path, body=doc)
 
         self.headers = {
-            'Client-ID': str(uuid.uuid4()),
+            'Client-ID': uuidutils.generate_uuid(),
         }
 
     def tearDown(self):
@@ -87,7 +86,7 @@ class TestMessagesMongoDB(base.V1Base):
         msg_ids = self._get_msg_ids(self.srmock.headers_dict)
         self.assertEqual(len(sample_messages), len(msg_ids))
 
-        expected_resources = [six.text_type(self.messages_path + '/' + id)
+        expected_resources = [str(self.messages_path + '/' + id)
                               for id in msg_ids]
         self.assertEqual(expected_resources, result_doc['resources'])
 
@@ -189,9 +188,6 @@ class TestMessagesMongoDB(base.V1Base):
 
         path = self.url_prefix + u'/queues/non-ascii-n\u0153me/messages'
 
-        if six.PY2:
-            path = path.encode('utf-8')
-
         self._post_messages(path)
         self.assertEqual(falcon.HTTP_400, self.srmock.status)
 
@@ -490,7 +486,7 @@ class TestMessagesFaultyDriver(base.V1BaseFaulty):
         path = self.url_prefix + '/queues/fizbit/messages'
         doc = '[{"body": 239, "ttl": 100}]'
         headers = {
-            'Client-ID': str(uuid.uuid4()),
+            'Client-ID': uuidutils.generate_uuid(),
         }
 
         self.simulate_post(path, project_id,
