@@ -14,11 +14,10 @@
 # limitations under the License.
 
 import datetime
-import json
+from unittest import mock
 
 import ddt
 import falcon
-import mock
 from oslo_serialization import jsonutils
 from oslo_utils import timeutils
 from oslo_utils import uuidutils
@@ -29,7 +28,7 @@ from zaqar.tests.unit.transport.wsgi import base
 
 
 @ddt.ddt
-class TestClaimsMongoDB(base.V2Base):
+class TestClaimsMongoDB(base.V1_1Base):
 
     config_file = 'wsgi_mongodb.conf'
 
@@ -47,12 +46,12 @@ class TestClaimsMongoDB(base.V2Base):
         self.claims_path = self.queue_path + '/claims'
         self.messages_path = self.queue_path + '/messages'
 
-        doc = json.dumps({"_ttl": 60})
+        doc = jsonutils.dumps({"_ttl": 60})
 
         self.simulate_put(self.queue_path, body=doc, headers=self.headers)
         self.assertEqual(falcon.HTTP_201, self.srmock.status)
 
-        doc = json.dumps({'messages': [{'body': 239, 'ttl': 300}] * 10})
+        doc = jsonutils.dumps({'messages': [{'body': 239, 'ttl': 300}] * 10})
         self.simulate_post(self.queue_path + '/messages',
                            body=doc, headers=self.headers)
         self.assertEqual(falcon.HTTP_201, self.srmock.status)
@@ -66,7 +65,6 @@ class TestClaimsMongoDB(base.V2Base):
 
         for db in storage.message_databases:
             connection.drop_database(db)
-
         self.simulate_delete(self.queue_path, headers=self.headers)
 
         super(TestClaimsMongoDB, self).tearDown()
@@ -92,7 +90,7 @@ class TestClaimsMongoDB(base.V2Base):
     def test_unacceptable_ttl_or_grace(self, ttl_grace):
         ttl, grace = ttl_grace
         self.simulate_post(self.claims_path,
-                           body=json.dumps({'ttl': ttl, 'grace': grace}),
+                           body=jsonutils.dumps({'ttl': ttl, 'grace': grace}),
                            headers=self.headers)
 
         self.assertEqual(falcon.HTTP_400, self.srmock.status)
@@ -102,7 +100,7 @@ class TestClaimsMongoDB(base.V2Base):
         href = self._get_a_claim()
 
         self.simulate_patch(href,
-                            body=json.dumps({'ttl': ttl}),
+                            body=jsonutils.dumps({'ttl': ttl}),
                             headers=self.headers)
 
         self.assertEqual(falcon.HTTP_400, self.srmock.status)
@@ -282,13 +280,13 @@ class TestClaimsMongoDB(base.V2Base):
         self.assertEqual(falcon.HTTP_204, self.srmock.status)
 
     def test_patch_nonexistent_claim_404s(self):
-        patch_data = json.dumps({'ttl': 100})
+        patch_data = jsonutils.dumps({'ttl': 100})
         self.simulate_patch(self.claims_path + '/a', body=patch_data,
                             headers=self.headers)
         self.assertEqual(falcon.HTTP_404, self.srmock.status)
 
 
-class TestClaimsFaultyDriver(base.V2BaseFaulty):
+class TestClaimsFaultyDriver(base.V1_1BaseFaulty):
 
     config_file = 'wsgi_faulty.conf'
 
