@@ -19,7 +19,7 @@ import re
 import uuid
 
 from oslo_utils import timeutils
-import six
+from urllib import parse as urllib_parse
 
 from zaqar.common import consts
 from zaqar.conf import transport
@@ -34,7 +34,7 @@ MIN_SUBSCRIPTION_TTL = 60
 _PURGBLE_RESOURCE_TYPES = {'messages', 'subscriptions'}
 # NOTE(kgriffs): Don't use \w because it isn't guaranteed to match
 # only ASCII characters.
-QUEUE_NAME_REGEX = re.compile('^[a-zA-Z0-9_\-.]+$')
+QUEUE_NAME_REGEX = re.compile(r'^[a-zA-Z0-9_\-.]+$')
 QUEUE_NAME_MAX_LEN = 64
 PROJECT_ID_MAX_LEN = 256
 
@@ -125,7 +125,7 @@ class Validator(object):
         if not pointer.startswith('/'):
             msg = _('Pointer `%s` does not start with "/".') % pointer
             raise ValidationFailed(msg)
-        if re.search('/\s*?/', pointer[1:]):
+        if re.search(r'/\s*?/', pointer[1:]):
             msg = _('Pointer `%s` contains adjacent "/".') % pointer
             raise ValidationFailed(msg)
         if len(pointer) > 1 and pointer.endswith('/'):
@@ -134,7 +134,7 @@ class Validator(object):
         if pointer[1:].strip() == '/':
             msg = _('Pointer `%s` does not contains valid token.') % pointer
             raise ValidationFailed(msg)
-        if re.search('~[^01]', pointer) or pointer.endswith('~'):
+        if re.search(r'~[^01]', pointer) or pointer.endswith('~'):
             msg = _('Pointer `%s` contains "~" not part of'
                     ' a recognized escape sequence.') % pointer
             raise ValidationFailed(msg)
@@ -341,6 +341,11 @@ class Validator(object):
                     msg, self._limits_conf.max_message_delay,
                     MIN_DELAY_TTL)
 
+        encrypted_queue = queue_metadata.get('_enable_encrypt_messages', False)
+        if encrypted_queue and not isinstance(encrypted_queue, bool):
+            msg = _(u'_enable_encrypt_messages must be boolean.')
+            raise ValidationFailed(msg)
+
         self._validate_retry_policy(queue_metadata)
 
     def queue_purging(self, document):
@@ -445,11 +450,12 @@ class Validator(object):
             raise ValidationFailed(
                 msg, self._limits_conf.max_messages_per_page)
 
-    def message_deletion(self, ids=None, pop=None):
+    def message_deletion(self, ids=None, pop=None, claim_ids=None):
         """Restrictions involving deletion of messages.
 
         :param ids: message ids passed in by the delete request
         :param pop: count of messages to be POPped
+        :param claim_ids: claim ids passed in by the delete request
         :raises ValidationFailed: if,
             pop AND id params are present together
             neither pop or id params are present
@@ -468,6 +474,13 @@ class Validator(object):
 
             raise ValidationFailed(msg)
 
+        if self._limits_conf.message_delete_with_claim_id:
+            if (ids and claim_ids is None) or (ids is None and claim_ids):
+                msg = _(u'The request should have both "ids" and "claim_ids" '
+                        'parameter in the request when '
+                        'message_delete_with_claim_id is True.')
+                raise ValidationFailed(msg)
+
         pop_uplimit = self._limits_conf.max_messages_per_claim_or_pop
         if pop is not None and not (0 < pop <= pop_uplimit):
             msg = _(u'Pop value must be at least 1 and may not '
@@ -556,7 +569,7 @@ class Validator(object):
         subscriber_type = None
 
         if subscriber:
-            parsed_uri = six.moves.urllib_parse.urlparse(subscriber)
+            parsed_uri = urllib_parse.urlparse(subscriber)
             subscriber_type = parsed_uri.scheme
 
             if subscriber_type not in self._limits_conf.subscriber_types:
@@ -668,3 +681,27 @@ class Validator(object):
                                        self._limits_conf.max_length_client_id)
         if self._limits_conf.client_id_uuid_safe == 'strict':
             uuid.UUID(client_id)
+
+    def topic_identification(self, topic, project):
+        """Restrictions on a project id & topic name pair.
+
+        :param queue: Name of the topic
+        :param project: Project id
+        :raises ValidationFailed: if the `name` is longer than 64
+            characters or contains anything other than ASCII digits and
+            letters, underscores, and dashes.  Also raises if `project`
+            is not None but longer than 256 characters.
+        """
+
+        if project is not None and len(project) > PROJECT_ID_MAX_LEN:
+            msg = _(u'Project ids may not be more than {0} characters long.')
+            raise ValidationFailed(msg, PROJECT_ID_MAX_LEN)
+
+        if len(topic) > QUEUE_NAME_MAX_LEN:
+            msg = _(u'Topic names may not be more than {0} characters long.')
+            raise ValidationFailed(msg, QUEUE_NAME_MAX_LEN)
+
+        if not QUEUE_NAME_REGEX.match(topic):
+            raise ValidationFailed(
+                _(u'Topic names may only contain ASCII letters, digits, '
+                  'underscores, and dashes.'))
